CREATE TABLE CONVERSATION
(
    ID            NUMERIC(18,0) NOT NULL IDENTITY,
    CONVERSATION_ID     VARCHAR(500) NOT NULL,
    SEQUENCE_OUT_BLOCKED  TINYINT NOT NULL,
    SEQUENCE_OUT_OFFSET     NUMERIC(18,0) NOT NULL,
    SEQUENCE_OFFSET     NUMERIC(18,0) NOT NULL,
    CONSTRAINT PK_CONVERSATION PRIMARY KEY (ID),
    CONSTRAINT UQ_CONVERSATION UNIQUE (CONVERSATION_ID)
);

CREATE TABLE MESSAGE
(
    ID                    NUMERIC(18,0) NOT NULL IDENTITY,
    MESSAGE_ID            VARCHAR(500) NOT NULL,
    FOLDER                VARCHAR(255) NULL,
    CREATION_TIME         NUMERIC(18,0) NULL,
    INBOUND               TINYINT NOT NULL,
    CONVERSATION          NUMERIC(18,0) NULL,
    SEQUENCE_NUMBER       NUMERIC(18,0) NULL,
    MESSAGE_TYPE          VARCHAR(100) NULL,
    SCHEMA_VERSION        VARCHAR(50) NULL,
    SCHEMA_SET            VARCHAR(50) NULL,
    PROTOCOL              VARCHAR(10) NULL,
    RECEIVER_ID           VARCHAR(30) NULL,
    SENDER_ID             VARCHAR(30) NULL,
    MESSAGE_TIME          NUMERIC(18,0) NULL,
    REFERENCE_ID          NUMERIC(18,0) NULL,
    TTL                   NUMERIC(18,0) NULL,
    PACKAGING_ID          VARCHAR(50) NULL,
    SIGNED_ACK_REQUESTED  TINYINT NULL,
    LOGINFO               VARCHAR(128) NULL,
    TESTFLAG              TINYINT NULL,
    STATUS                NUMERIC(2,0) NULL,
    ACK_RECEIVED          TINYINT NULL,
    ADAPTER_ID            VARCHAR(50) NULL,
    MESSENGER_ID          NUMERIC(3,0) NULL,
    CONSTRAINT PK_MESSAGE PRIMARY KEY (ID),
    CONSTRAINT UQ_MESSAGE UNIQUE (MESSAGE_ID, INBOUND),
    CONSTRAINT FK_MESSAGE_CONVERSATION FOREIGN KEY (CONVERSATION) REFERENCES CONVERSATION(ID),
    CONSTRAINT FK_MESSAGE_MESSAGE FOREIGN KEY (REFERENCE_ID) REFERENCES MESSAGE(ID)
);

create index IDX_MESSAGE_CONVERSATION ON MESSAGE (CONVERSATION);
create index IDX_MESSAGE_REFID ON MESSAGE (REFERENCE_ID);
create index IDX_MESSAGE_MESSAGE_TIME ON MESSAGE (MESSAGE_TIME);


CREATE TABLE FILE_REF
(
    ID                    NUMERIC(18,0) NOT NULL IDENTITY,
    MESSAGE_ID            NUMERIC(18,0) NOT NULL,
    FILENAME              VARCHAR(255) NOT NULL,
    CONSTRAINT PK_FILE_REF PRIMARY KEY (ID),
    CONSTRAINT UQ_FILE_REF UNIQUE (MESSAGE_ID, FILENAME),
    CONSTRAINT FK_FILE_REF_MESSAGE FOREIGN KEY (MESSAGE_ID) REFERENCES MESSAGE(ID)
);

create index IDX_FILE_REF_MSGID ON FILE_REF (MESSAGE_ID);

CREATE TABLE LOG_MESSAGE
(
    ID                     INTEGER NOT NULL,
    TYPE                   INTEGER NOT NULL,
    BEGIN_COMPONENT        INTEGER NOT NULL,
    END_COMPONENT          INTEGER NOT NULL,
    TEXT                   VARCHAR(255) NOT NULL,
    CONSTRAINT PK_LOG_MESSAGE PRIMARY KEY(ID)
);

CREATE TABLE MESSAGE_LOG
(
    ID                    NUMERIC(18,0) NOT NULL,
    TIMESTAMP             NUMERIC(18,0) NOT NULL,
    MESSAGE_ID            NUMERIC(18,0) NOT NULL,
    TEXT_ID               INTEGER NULL,
    ADDITIONAL_TEXT       VARCHAR(512) NULL,
    CONSTRAINT PK_MESSAGE_LOG PRIMARY KEY (ID),
    CONSTRAINT FK_MESSAGE_LOG_MESSAGE FOREIGN KEY (MESSAGE_ID) REFERENCES MESSAGE(ID),
    CONSTRAINT FK_MESSAGE_LOG_LOG_MESSAGE FOREIGN KEY (TEXT_ID) REFERENCES LOG_MESSAGE(ID)
);
CREATE INDEX IDX_MSGLOG_MSGID ON MESSAGE_LOG (MESSAGE_ID);
create index IDX_MSGLOG_TIMESTAMP ON MESSAGE_LOG (TIMESTAMP);

CREATE TABLE INBOUND_QUEUE
(
    MESSAGE_ID            NUMERIC(18,0) NOT NULL,
    TIMESECONDS           NUMERIC(18,0) NULL,
    ADAPTER_ID            VARCHAR(50) NULL,
    NEXT_TRANSMISSION     NUMERIC(18,0) NULL,
    MESSENGER_ID      NUMERIC(3,0) NULL,
    ACK_SENT              TINYINT NULL,
    CONSTRAINT PK_INBOUND_QUEUE PRIMARY KEY (MESSAGE_ID),
    CONSTRAINT FK_INBOUND_QUEUE_MESSAGE FOREIGN KEY (MESSAGE_ID) REFERENCES MESSAGE(ID)
);

CREATE TABLE OUTBOUND_QUEUE
(
    MESSAGE_ID             NUMERIC(18,0) NOT NULL,
    TIMESECONDS            NUMERIC(18,0) NULL,
    RECEIVER_URL           VARCHAR(512) NULL,
    PACKAGING              VARCHAR(20) NULL,
    TIME_TO_LIVE           NUMERIC(18,0) NULL,
    RETRY_INTERVAL         INTEGER NULL,
    RETRIES          INTEGER NULL,
    PARALLEL_DELIVERIES    INTEGER NULL,
    ACK_REQUIRED           TINYINT NULL,
    SENDER_ID              VARCHAR(30) NULL,
    RECEIVER_ID            VARCHAR(30) NULL,
    LAST_TRANSMISSION      NUMERIC(18,0) NULL,
    MESSENGER_ID       NUMERIC(3,0) NULL,
    CONSTRAINT PK_OUTBOUND_QUEUE PRIMARY KEY (MESSAGE_ID),
    CONSTRAINT FK_OUTBOUND_QUEUE_MESSAGE FOREIGN KEY (MESSAGE_ID) REFERENCES MESSAGE(ID)
);

CREATE TABLE MDNDATA
(
    MESSAGE_ID             NUMERIC(18,0) NOT NULL,
    DIGEST                 VARCHAR(512) NULL,
    DIGEST_ALGORITHM       VARCHAR(30) NULL,
    CONSTRAINT PK_MDNDATA PRIMARY KEY (MESSAGE_ID),
    CONSTRAINT FK_MDNDATA_MESSAGE FOREIGN KEY (MESSAGE_ID) REFERENCES MESSAGE(ID)
);

CREATE TABLE ADAPTER_INFO
(
    ADAPTER_ID             VARCHAR(50) NOT NULL,
    MESSENGER_ID       NUMERIC(3,0) NOT NULL,
    ADDRESS                VARCHAR(128) NULL,
    MAXIMUM_THREADS        INTEGER NULL,
    PROCESSING_TIMEOUT     INTEGER NULL,
    FEATURES               VARCHAR(255) NULL,
    CONSTRAINT PK_ADAPTER_INFO PRIMARY KEY(ADAPTER_ID, MESSENGER_ID)
);
