CREATE TABLE CONVERSATION
(
    ID            BIGINT NOT NULL AUTO_INCREMENT,
    CONVERSATION_ID     VARCHAR(500) NOT NULL,
    SEQUENCE_OUT_BLOCKED  INTEGER(1) NOT NULL,
    SEQUENCE_OUT_OFFSET     BIGINT NOT NULL,
    SEQUENCE_OFFSET     BIGINT NOT NULL,
    CONSTRAINT PK_CONVERSATION PRIMARY KEY (ID),
    CONSTRAINT UQ_CONVERSATION UNIQUE (CONVERSATION_ID)
)ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE MESSAGE (
    ID                      BIGINT NOT NULL AUTO_INCREMENT,
    MESSAGE_ID        VARCHAR(500) NOT NULL,
    FOLDER                  VARCHAR(255) NULL,
    CREATION_TIME           BIGINT NULL,
    INBOUND         INTEGER(1) NOT NULL,
    CONVERSATION            BIGINT NULL,
    SEQUENCE_NUMBER         BIGINT NULL,
    MESSAGE_TYPE      VARCHAR(100) NULL,
    SCHEMA_VERSION      VARCHAR(50) NULL,
    SCHEMA_SET        VARCHAR(50) NULL,
    PROTOCOL        VARCHAR(10) NULL,
    RECEIVER_ID       VARCHAR(30) NULL,
    SENDER_ID       VARCHAR(30) NULL,
    MESSAGE_TIME      BIGINT NULL,
    REFERENCE_ID      BIGINT NULL,
    TTL                     BIGINT NULL,
    PACKAGING_ID            VARCHAR(50) NULL,
    SIGNED_ACK_REQUESTED    INTEGER(1) NULL,
    LOGINFO         VARCHAR(128) NULL,
    TESTFLAG        INTEGER(1) NULL,
    STATUS          INTEGER(2) NULL,
    ACK_RECEIVED        INTEGER(1) NULL,
    ADAPTER_ID              VARCHAR(50) NULL,
    MESSENGER_ID            INTEGER(3) NULL,
    CONSTRAINT PK_MESSAGE PRIMARY KEY (ID),
    CONSTRAINT UQ_MESSAGE UNIQUE (MESSAGE_ID, INBOUND),
    CONSTRAINT FK_MES_CON FOREIGN KEY (CONVERSATION) REFERENCES CONVERSATION(ID),
    CONSTRAINT FK_MESSAGE_MESSAGE FOREIGN KEY (REFERENCE_ID) REFERENCES MESSAGE(ID)
)ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE INDEX IDX_MESSAGE_CONVERSATION ON MESSAGE (CONVERSATION);
CREATE INDEX IDX_MESSAGE_REFID ON MESSAGE (REFERENCE_ID);
CREATE INDEX IDX_MESSAGE_MESSAGE_TIME ON MESSAGE (MESSAGE_TIME);

CREATE TABLE FILE_REF (
    ID                      BIGINT NOT NULL AUTO_INCREMENT,
    MESSAGE_ID              BIGINT NOT NULL,
    FILENAME                VARCHAR(255) NOT NULL,
    CONSTRAINT PK_FILE_REF PRIMARY KEY (ID),
    CONSTRAINT UQ_FILE_REF UNIQUE (MESSAGE_ID, FILENAME),
    CONSTRAINT FK_FILE_REF_MESSAGE FOREIGN KEY (MESSAGE_ID) REFERENCES MESSAGE(ID)
)ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE INDEX IDX_FILE_REF_MSGID ON FILE_REF (MESSAGE_ID);

CREATE TABLE LOG_MESSAGE (
    ID                     INTEGER(6) NOT NULL,
    TYPE                   INTEGER(1) NOT NULL,
    BEGIN_COMPONENT        INTEGER(1) NOT NULL,
    END_COMPONENT          INTEGER(1) NOT NULL,
    TEXT                   VARCHAR(255) NOT NULL,
    CONSTRAINT PK_LOG_MESSAGE PRIMARY KEY (ID)
)ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE MESSAGE_LOG (
    ID                      BIGINT NOT NULL,
    TIMESTAMP       BIGINT NOT NULL,
    MESSAGE_ID        BIGINT NOT NULL,
    TEXT_ID         INTEGER(6) NULL,
    ADDITIONAL_TEXT     TEXT NULL,
    CONSTRAINT PK_MESSAGE_LOG PRIMARY KEY (ID),
    CONSTRAINT FK_MESSAGE_LOG_MESSAGE FOREIGN KEY (MESSAGE_ID) REFERENCES MESSAGE(ID),
    CONSTRAINT FK_MESSAGE_LOG_LOG_MESSAGE FOREIGN KEY (TEXT_ID) REFERENCES LOG_MESSAGE(ID)
)ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE INDEX IDX_MSGLOG_MSGID ON MESSAGE_LOG (MESSAGE_ID);
CREATE INDEX IDX_MSGLOG_TIMESTAMP ON MESSAGE_LOG (TIMESTAMP);

CREATE TABLE INBOUND_QUEUE (
    MESSAGE_ID        BIGINT NOT NULL,
    TIMESECONDS       BIGINT NULL,
    ADAPTER_ID        VARCHAR(50) NULL,
    NEXT_TRANSMISSION   BIGINT NULL,
    MESSENGER_ID      INTEGER(3) NULL,
    ACK_SENT                INTEGER(3) NULL,
    CONSTRAINT PK_INBOUND_QUEUE PRIMARY KEY (MESSAGE_ID),
    CONSTRAINT FK_INBOUND_QUEUE_MESSAGE FOREIGN KEY (MESSAGE_ID) REFERENCES MESSAGE(ID)
)ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE OUTBOUND_QUEUE (
    MESSAGE_ID        BIGINT NOT NULL,
    TIMESECONDS       BIGINT NULL,
    RECEIVER_URL      VARCHAR(512) NULL,
    PACKAGING       VARCHAR(20) NULL,
    TIME_TO_LIVE      BIGINT NULL,
    RETRY_INTERVAL      INTEGER(10) NULL,
    RETRIES         INTEGER(3) NULL,
    PARALLEL_DELIVERIES     INTEGER(10) NULL,
    ACK_REQUIRED      INTEGER(1) NULL,
    SENDER_ID               VARCHAR(30) NULL,
    RECEIVER_ID             VARCHAR(30) NULL,
    LAST_TRANSMISSION   BIGINT NULL,
    MESSENGER_ID      INTEGER(3) NULL,
    CONSTRAINT PK_OUTBOUND_QUEUE PRIMARY KEY (MESSAGE_ID),
    CONSTRAINT FK_OUTBOUND_QUEUE_MESSAGE FOREIGN KEY (MESSAGE_ID) REFERENCES MESSAGE(ID)
)ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE MDNDATA
(
    MESSAGE_ID              BIGINT NOT NULL,
    DIGEST                  TEXT NULL,
    DIGEST_ALGORITHM        VARCHAR(30) NULL,
    CONSTRAINT PK_MDNDATA PRIMARY KEY (MESSAGE_ID),
    CONSTRAINT FK_MDNDATA_MESSAGE FOREIGN KEY (MESSAGE_ID) REFERENCES MESSAGE(ID)
)ENGINE=InnoDB DEFAULT CHARSET=latin1;

CREATE TABLE ADAPTER_INFO (
    ADAPTER_ID        VARCHAR(50) NOT NULL,
    MESSENGER_ID      INTEGER(3) NOT NULL,
    ADDRESS         VARCHAR(128) NULL,
    MAXIMUM_THREADS     INTEGER(6) NULL,
    PROCESSING_TIMEOUT      INTEGER(10) NULL,
    FEATURES                VARCHAR(255) NULL,
    CONSTRAINT PK_ADAPTER_INFO PRIMARY KEY (ADAPTER_ID, MESSENGER_ID)
)ENGINE=InnoDB DEFAULT CHARSET=latin1;
